"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeLineCapStyle = exports.makePathsFromCommands = void 0;
const point_1 = require("./point");
const curvePoint_1 = require("./curvePoint");
function makePath(curvePoints, isClosed) {
    return {
        isClosed,
        points: curvePoints,
    };
}
// Points are normalized between 0 and 1, relative to the frame.
// We use the original frame here and can scale it later.
//
// This is a rough port of Lona's PDF to Sketch path conversion
// https://github.com/airbnb/Lona/blob/94fd0b26de3e3f4b4496cdaa4ab31c6d258dc4ac/studio/LonaStudio/Utils/Sketch.swift#L285
function makePathsFromCommands(commands, frame) {
    const paths = [];
    let curvePoints = [];
    function finishPath(isClosed) {
        if (curvePoints.length === 0)
            return;
        const path = makePath(curvePoints, isClosed);
        paths.push(path);
        curvePoints = [];
    }
    commands.forEach((command) => {
        const { type, data } = command;
        switch (type) {
            case 'move': {
                finishPath(false);
                const { to } = data;
                const curvePoint = curvePoint_1.makeCurvePoint(point_1.normalizePointInRect(to, frame), undefined, undefined, 1);
                curvePoints.push(curvePoint);
                break;
            }
            case 'line': {
                const { to } = data;
                const curvePoint = curvePoint_1.makeCurvePoint(point_1.normalizePointInRect(to, frame), undefined, undefined, 1);
                curvePoints.push(curvePoint);
                break;
            }
            case 'cubicCurve': {
                const { to, controlPoint1, controlPoint2 } = data;
                if (curvePoints.length > 0) {
                    const last = curvePoints[curvePoints.length - 1];
                    last.curveFrom = curvePoint_1.describePoint(point_1.normalizePointInRect(controlPoint1, frame));
                    last.curveMode = 2;
                    last.hasCurveFrom = true;
                }
                const curvePoint = curvePoint_1.makeCurvePoint(point_1.normalizePointInRect(to, frame), undefined, point_1.normalizePointInRect(controlPoint2, frame), 2);
                curvePoints.push(curvePoint);
                break;
            }
            case 'close': {
                // If first and last points are equal, combine them
                if (curvePoints.length > 0) {
                    const first = curvePoints[0];
                    const last = curvePoints[curvePoints.length - 1];
                    if (first.point == last.point && last.hasCurveTo) {
                        first.curveTo = last.curveTo;
                        first.hasCurveTo = last.hasCurveTo;
                        first.curveMode = 2;
                        curvePoints.pop();
                    }
                }
                finishPath(true);
                break;
            }
            default:
                throw new Error(`Invalid SVG path command: ${type}`);
        }
    });
    finishPath(false);
    return paths;
}
exports.makePathsFromCommands = makePathsFromCommands;
function makeLineCapStyle(strokeLineCap) {
    switch (strokeLineCap) {
        case 'butt':
            return 0;
        case 'round':
            return 1;
        case 'square':
            return 2;
        default:
            throw new Error(`Invalid SVG stroke line cap: ${strokeLineCap}`);
    }
}
exports.makeLineCapStyle = makeLineCapStyle;
