"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseStyle = exports.makeStyle = exports.makeShadow = exports.makeBorderOptions = void 0;
const sketch_file_format_ts_1 = require("@sketch-hq/sketch-file-format-ts");
const models_1 = require("./models");
const hasAnyDefined_1 = require("../utils/hasAnyDefined");
const borders_1 = require("./borders");
const DEFAULT_SHADOW_COLOR = '#000';
const SHADOW_STYLES = [
    'shadowColor',
    'shadowOffset',
    'shadowOpacity',
    'shadowRadius',
    'shadowSpread',
    'textShadowColor',
    'textShadowOffset',
    'textShadowOpacity',
    'textShadowRadius',
    'textShadowSpread',
];
const makeDashPattern = (style, width) => {
    switch (style) {
        case 'dashed':
            return [width * 3, width * 3];
        case 'dotted':
            return [width, width];
        case 'solid':
            return [];
        default:
            return [];
    }
};
exports.makeBorderOptions = (style, width, lineCapStyle = sketch_file_format_ts_1.FileFormat1.LineCapStyle.Butt, lineJoinStyle = sketch_file_format_ts_1.FileFormat1.LineJoinStyle.Miter) => ({
    _class: 'borderOptions',
    isEnabled: false,
    dashPattern: makeDashPattern(style, width),
    lineCapStyle,
    lineJoinStyle,
});
exports.makeShadow = (style) => {
    const opacity = style.shadowOpacity !== undefined && style.shadowOpacity !== null
        ? style.shadowOpacity
        : 'textShadowOpacity' in style &&
            style.textShadowOpacity !== undefined &&
            style.textShadowOpacity !== null
            ? style.textShadowOpacity
            : 1;
    const color = style.shadowColor ||
        ('textShadowColor' in style && style.textShadowColor) ||
        DEFAULT_SHADOW_COLOR;
    const radius = style.shadowRadius !== undefined && style.shadowRadius !== null
        ? style.shadowRadius
        : 'textShadowRadius' in style &&
            style.textShadowRadius !== undefined &&
            style.textShadowRadius !== null
            ? style.textShadowRadius
            : 1;
    const spread = style.shadowSpread !== undefined && style.shadowSpread !== null
        ? style.shadowSpread
        : 'textShadowSpread' in style &&
            style.textShadowSpread !== undefined &&
            style.textShadowSpread !== null
            ? style.textShadowSpread
            : 1;
    let { width: offsetX, height: offsetY } = style.shadowOffset || ('textShadowOffset' in style && style.textShadowOffset) || {};
    if (!offsetX) {
        offsetX = 0;
    }
    if (!offsetY) {
        offsetY = 0;
    }
    const commonProps = {
        isEnabled: true,
        blurRadius: radius,
        color: models_1.makeColorFromCSS(color, opacity),
        contextSettings: {
            _class: 'graphicsContextSettings',
            blendMode: sketch_file_format_ts_1.FileFormat1.BlendMode.Normal,
            opacity: 1,
        },
        offsetX,
        offsetY,
        spread,
    };
    if (style.shadowInner) {
        return Object.assign({ _class: 'innerShadow' }, commonProps);
    }
    return Object.assign({ _class: 'shadow' }, commonProps);
};
exports.makeStyle = (style, fills, shadowsProp) => {
    const json = {
        _class: 'style',
        fills: [],
        miterLimit: 10,
        innerShadows: [],
        shadows: [],
        borderOptions: exports.makeBorderOptions(borders_1.DEFAULT_BORDER_STYLE, 0, 0, 0),
        startMarkerType: sketch_file_format_ts_1.FileFormat1.MarkerType.OpenArrow,
        endMarkerType: sketch_file_format_ts_1.FileFormat1.MarkerType.OpenArrow,
        windingRule: sketch_file_format_ts_1.FileFormat1.WindingRule.EvenOdd,
        colorControls: {
            _class: 'colorControls',
            isEnabled: false,
            brightness: 1,
            contrast: 1,
            hue: 1,
            saturation: 1,
        },
    };
    if (fills && fills.length) {
        json.fills = (json.fills || []).concat(fills);
    }
    if (!style) {
        return json;
    }
    if (style.opacity) {
        json.contextSettings = {
            _class: 'graphicsContextSettings',
            blendMode: sketch_file_format_ts_1.FileFormat1.BlendMode.Normal,
            opacity: style.opacity,
        };
    }
    if (style.backgroundColor) {
        const fill = models_1.makeColorFill(style.backgroundColor);
        (json.fills || []).unshift(fill);
    }
    if (hasAnyDefined_1.hasAnyDefined(style, SHADOW_STYLES)) {
        const shadow = [exports.makeShadow(style)];
        if (style.shadowInner) {
            json.innerShadows = shadow;
        }
        else {
            json.shadows = shadow;
        }
    }
    if (shadowsProp) {
        shadowsProp.forEach((shadowStyle) => {
            if (!shadowStyle) {
                return;
            }
            const shadow = exports.makeShadow(shadowStyle);
            if (shadowStyle.shadowInner) {
                (json.innerShadows || []).push(shadow);
            }
            else {
                (json.shadows || []).push(shadow);
            }
        });
    }
    return json;
};
function parseStyle(json) {
    const style = {};
    if (json.contextSettings && json.contextSettings.opacity !== 1) {
        style.opacity = json.contextSettings.opacity;
    }
    if (json.fills &&
        json.fills.length > 0 &&
        json.fills[0].fillType === sketch_file_format_ts_1.FileFormat1.FillType.Color &&
        json.fills[0].isEnabled) {
        const color = json.fills[0].color;
        style.backgroundColor = `#${Math.round(color.red * 255).toString(16)}${Math.round(color.green * 255).toString(16)}${Math.round(color.blue * 255).toString(16)}`;
        if (color.alpha !== 1) {
            style.backgroundColor += `${Math.round(color.alpha * 255).toString(16)}`;
        }
    }
    if ((json.shadows && json.shadows.length > 0 && json.shadows[0].isEnabled) ||
        (json.innerShadows && json.innerShadows.length > 0 && json.innerShadows[0].isEnabled)) {
        const isNormalShadow = json.shadows && json.shadows.length > 0 && json.shadows[0].isEnabled;
        const shadow = isNormalShadow ? (json.shadows || [])[0] : (json.innerShadows || [])[0];
        style.shadowRadius = shadow.blurRadius;
        style.shadowSpread = shadow.spread;
        style.shadowOffset = {
            width: shadow.offsetX,
            height: shadow.offsetY,
        };
        const color = shadow.color;
        style.shadowColor = `#${Math.round(color.red * 255).toString(16)}${Math.round(color.green * 255).toString(16)}${Math.round(color.blue * 255).toString(16)}`;
        if (color.alpha !== 1) {
            style.shadowOpacity = color.alpha;
        }
        if (!isNormalShadow) {
            style.shadowInner = true;
        }
    }
    return style;
}
exports.parseStyle = parseStyle;
